import logging

__version__ = '1.0.0'

handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('[%(levelname)s] %(message)s'))
logger = logging.getLogger('rupayamn')
logger.addHandler(handler)
logger.setLevel('CRITICAL')



import logging
import sys

from clint import resources
from slugify import slugify
import docker

from rupayamn import display
from rupayamn.elements.network import Network
from rupayamn.elements.service import Service
from rupayamn.elements.volume import Volume
from rupayamn.environments import environments

logger = logging.getLogger('rupayamn')
resources.init('rupaya', 'rupayamn')


class Configuration:
    """docstring for Configuration."""

    def __init__(self, name: str = None, net: str = None,
                 pkey: str = None, start: bool = False,
                 docker_url: str = None, api: bool = False) -> None:
        self.networks = {}
        self.services = {}
        self.volumes = {}
        self.name = name
        self.net = net
        self.pkey = pkey or ''
        self.force_recreate = False
        if not docker_url:
            self.client = docker.from_env()
        else:
            self.client = docker.DockerClient(base_url=docker_url)
        self.api = 'True' if api else 'False'
        try:
            self.client.ping()
        except Exception as e:
            logger.error(e)
            display.error_docker()
            sys.exit('\n')
        if resources.user.read('name'):
            self._load()
        elif start:
            self._write()
        else:
            display.error_start_not_initialized()
            sys.exit('\n')
        self._compose()

    def _load(self) -> None:
        if self.name or self.net or self.pkey:
            display.warning_ignoring_start_options(resources.user.read('name'))
        self.name = resources.user.read('name')
        self.net = resources.user.read('net')
        self.api = resources.user.read('api')

    def _write(self) -> None:
        if not self.name:
            display.error_start_option_required('--name')
            sys.exit('\n')
        elif not self.net:
            display.error_start_option_required('--net')
            sys.exit('\n')
        elif not self.pkey:
            display.error_start_option_required('--pkey')
            sys.exit('\n')
        self._validate()
        resources.user.write('name', self.name)
        resources.user.write('net', self.net)
        resources.user.write('api', self.api)

    def _compose(self) -> None:
        self.networks['rupayamn'] = Network(
            name='{}_rupayamn'.format(self.name),
            client=self.client
        )
        self.volumes['chaindata'] = Volume(
            name='{}_chaindata'.format(self.name),
            client=self.client
        )
        if self.net == 'mainnet':
            tag = 'stable'
        elif self.net == 'testnet':
            tag = 'testnet'
        else:
            tag = 'latest'
        if self.api == 'True':  # this is dirty, should be refactored
            rupaya_ports = {'10707/udp': 10707, '10707/tcp': 10707,
                            '7050/tcp': 7050, '8050/tcp': 8050}
        else:
            rupaya_ports = {'10707/udp': 10707, '10707/tcp': 10707}
        self.services['rupaya'] = Service(
            name='{}_rupaya'.format(self.name),
            image='rupaya/node:{}'.format(
                'devnet' if tag == 'latest' else tag
            ),
            network=self.networks['rupayamn'].name,
            environment={
                'IDENTITY': '{}'.format(self.name),
                'PRIVATE_KEY': '{}'.format(self.pkey)
            },
            volumes={
                self.volumes['chaindata'].name: {
                    'bind': '/rupaya', 'mode': 'rw'
                }
            },
            ports=rupaya_ports,
            client=self.client
        )
        for container, variables in environments[self.net].items():
            for variable, value in variables.items():
                self.services[container].add_environment(
                    name=variable,
                    value=value
                )

    def _validate(self) -> None:
        self.name = slugify(self.name)
        if len(self.name) < 5 or len(self.name) > 30:
            display.error_validation_option('--name', '5 to 30 characters '
                                            'slug')
            sys.exit('\n')
        if len(self.pkey) != 64:
            display.error_validation_option('--pkey', '64 characters hex '
                                            'string')
            sys.exit('\n')
        try:
            bytes.fromhex(self.pkey)
        except ValueError:
            display.error_validation_option('--pkey', '64 characters hex '
                                            'string')
            sys.exit('\n')

    def remove(self) -> None:
        resources.user.delete('name')
        resources.user.delete('net')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rupayamn', 'rupayamn.elements']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'clint>=0.5.1,<0.6.0',
 'docker>=3.5.0,<4.0.0',
 'pastel>=0.1.0,<0.2.0',
 'python-slugify>=1.2,<2.0']

entry_points = \
{'console_scripts': ['rupayamn = rupayamn.rupayamn:main']}

setup_kwargs = {
    'name': 'rupayamn',
    'version': '1.0.5',
    'description': 'Quickstart your masternode',
    'long_description': "Rupaya MasterNode (rupayamn) is a cli tool to help you run a Rupaya masternode\n\n## Running and applying a masternode\n\nIf you are consulting this repo, it's probably because you want to run a masternode.\nFor complete guidelines on running a masternode candidate, please refer to the [documentation](https://docs.rupx.io/masternode/requirements/).\n",
    'author': 'Aasim',
    'author_email': 'aasim@rupx.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://rupx.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
